/* $Id: mdi_create_time_vtree.sas,v 1.15.2.10.6.2 2009/04/30 15:05:32 sugupt Exp $ */
/*-----------------------------------------------------------------------------------------------------------------------------
Description: 
Input: sasdata.primary_timeid_list, sasdata.&db_tablename&db_tablename_suffix._hasdata_timeids 
Output: sasdata.mdi_time_vtree
0--------1---------2--------3----------4--------5----------6---------7---------8---------9---------0---------1---------2-----*/

%timestamp(b,c_time_vtree)
%include sascode(mdi_define_level_mvars_time);             /* uses datamgr.fact_table, maxapp.mmax_config, maxdata.path_seg, maxdata.hier_level */
%include sascode(mdi_create_time_htree);                   /* uses maxapp.lv5time */
%define_primary_timeid_mvars                               /* uses sasdata.primary_timeid_list */
%macro _parent_numbers;
   %local i j lev plev num pnum;
   %do i=0 %to &ntln;
      %global tpn&i; %let tpn&i=0;
   %end;
   %do i=2 %to &ntlev;
      %let j=%eval(&i-1);
      %let lev =&&tlev&i;
      %let plev=&&tlev&j;
      %let num =&&tln&lev;
      %let pnum=&&tln&plev;
      %let tpn&num=&pnum;
      %if %str(&pnum)^=%str(0) %then
         %do;
            %logger(m,info,%bquote([mdi_create_time_vtree] time number &num (time_level=&&tnl&num) has a parent time number of &pnum (time_level=&&tnl&pnum)))
         %end;
   %end;
%mend _parent_numbers;
%_parent_numbers
data primary_timeid_vtree;
   merge
      sasdata.primary_timeid_list(in=in1)                  /* contains time_level, time_id, primary_timeid_n */
      sasdata.mdi_time_htree
      ;
   by time_level time_id;
   if in1;
   array time_i{5} time_id1   -time_id5;
   array time_l{5} time_level1-time_level5;
   do i=1 to dim(time_i);
      time_id   =time_i{i};
      time_level=time_l{i};
      if time_id>0 then output;
   end;
   drop i;
run;
proc summary data=primary_timeid_vtree nway;
   class time_level time_id;
   output out=primary_timeid_vtree(compress=no drop=_type_ _freq_);
run;
data sasdata.primary_timeid_vtree;
   merge
      primary_timeid_vtree
      sasdata.primary_timeid_list
      ;
   by time_level time_id;
   if first.time_id;
run;
data sasdata.primary_hasdata_timeids;
   merge
      sasdata.&db_tablename&db_tablename_suffix._hasdata_timeids(in=in1)
      sasdata.primary_timeid_vtree(in=in2)
      ;
   by time_level time_id;
   if in1 then hasdata=1;
*   put %logger(d,info,sasdata.primary_hasdata_timeids) time_level= z2. time_id= 9.0 primary_timeid_n= hasdata=;
run;
proc print data=sasdata.primary_hasdata_timeids uniform;
   title 'R039: mdi_create_time_vtree.sas: sasdata.primary_hasdata_timeids';
run;
title;
%macro create_time_vtree(primary_timeid_n=,primary_time_level=,primary_time_id=,outtab=);
   /*------------------------------------------------------------------------------------------------+
   | given a particular value for &primary_time_level and &primary_time_id, create a dataset         |
   | called &outtab containing 1 row for each time_id/time_level value that needs to be downloaded   |
   | from the main MAXDATA.MFINC table so that aggregations can be done.                             |
   | this does not do any aggregation; it just makes a list of the time values you need to download. |
   +------------------------------------------------------------------------------------------------*/

   %logger(m,info,%bquote([create_time_vtree] begin primary_time_level=&primary_time_level primary_time_id=&primary_time_id))

   /* the results of this macro will be to create a table containing the following sort of information:

   recomputed computing_final_data base_id parent_row time_level time_id time_type time_parent_level time_parent_id days_in_period start_date end_date   time_name  bop_level bop_id eop_level eop_id
      1              0                0       1          47           35    Year       .                  .            364         2003-12-01 2004-11-28 2004         48          69   48         70
      0              0                0       0          48           69    Season    47                 35            182         2003-12-01 2004-05-30 Spring        .           .    .          .
      1              1                0       1          48           70    Season    47                 35            182         2004-05-31 2004-11-28 Fall         49         140   49        141
      0              0                0       0          49          140    Quarter   48                 70             91         2004-05-31 2004-08-29 Q3            .           .    .          .
      1              1                0       1          49          141    Quarter   48                 70             91         2004-08-30 2004-11-28 Q4           50         421   50        423
      0              0                0       0          50          421    Month     49                141             35         2004-08-30 2004-10-03 Sep-04        .           .    .          .
      0              0                0       0          50          422    Month     49                141             28         2004-10-04 2004-10-31 Oct-04        .           .    .          .
      1              1                0       1          50          423    Month     49                141             28         2004-11-01 2004-11-28 Nov-04       51        1837   51       1839
      0              0                0       0          51         1837    Week      50                423              7         2004-11-01 2004-11-07 Nov-01-04     .           .    .          .
      0              0                0       0          51         1838    Week      50                423              7         2004-11-08 2004-11-14 Nov-08-04     .           .    .          .
      1              1                1       0          51         1839    Week      50                423              7         2004-11-15 2004-11-21 Nov-15-04     .           .    .          .

   */

   %local i j;

   /* time_level means 47 to 51; time_number means 1 to 5 */
   %local primary_time_number; %let primary_time_number=&&tln&primary_time_level;

   %global valid_time_id; %let valid_time_id=N;

   %global this_id parent_id;

   %local this_number this_level parent_number parent_level grandparent_number grandparent_level;
   %let this_id           =&primary_time_id;
   %let this_number       =&primary_time_number;
   %let this_level        =&&tnl&this_number;
   %let parent_number     =&&tpn&this_number;
   
   %save_syscc /* todogander -no grandparent level when loading at season level, so masking syscc */
   
   %if &parent_number>0 %then
      %do;
         %let parent_level      =&&tnl&parent_number;
         %let grandparent_number=&&tpn&parent_number;
         %let grandparent_level =&&tnl&grandparent_number;
      %end;
   %else
      %do;
         %let parent_level      =0;
         %let grandparent_number=0;
         %let grandparent_level =0;
      %end;

   %restore_saved_syscc /* todogander */

   /*----------------------------------------------------------------------------------------+
   | note that time_id values in time_vtree where parent_row=1 will not be used to           |
   | download records from the database because those rows can be computed by aggregating    |
   | their child rows.  it is only the siblings (if any) of the parent rows that need to be  |
   | downloaded.                                                                             |
   +----------------------------------------------------------------------------------------*/

   data time_vtree;
      retain
         parent_row sibling
         primary_time_level primary_time_id
         time_level         time_id
         time_type
         time_parent_level  time_parent_id
         this_days_in_period
         this_start_date this_start_date_time
         this_end_date   this_end_date_time
         ;
      set maxapp.lv&primary_time_number.time(
                                             where=(time_id=&this_id)
                                             rename=(
                                                     lv&this_number.time_lkup_id   =time_id
                                                     lv&this_number.time_type      =time_type
                                                     lv&this_number.time_start_date=this_start_date_time
                                                     lv&this_number.time_end_date  =this_end_date_time
                                                     %if &parent_number>0 %then
                                                        %do;
                                                     lv&parent_number.time_lkup_id =time_parent_id
                                                        %end;
                                                     days_in_period                =this_days_in_period
                                                    )
                                             keep=
                                                     lv&this_number.time_lkup_id
                                                     lv&this_number.time_type
                                                     lv&this_number.time_start_date
                                                     lv&this_number.time_end_date
                                                     %if &parent_number>0 %then
                                                        %do;
                                                     lv&parent_number.time_lkup_id
                                                        %end;
                                                     time_name
                                                     days_in_period
                                            );
      parent_row=0;
      sibling=0;
      primary_time_id   =&primary_time_id;
      primary_time_level=&primary_time_level;
      time_level=&this_level;
      time_parent_level=&parent_level;
      time_parent_id=time_parent_id;
      if time_parent_level<&tlevy then call missing(time_parent_level);
       /* Subhash (S0447415). Check of the date value aganst 22jan2222 is replaced with retrieving data types using vformat syntax --Subhash */ 
      /* ??? ??? the use of vformat below needs to be changed to checking the value of the date variable itself */
      this_start_date=this_start_date_time; if index(vformat(this_start_date_time),'DATETIME')>0 then this_start_date=datepart(this_start_date_time); 
      this_end_date  =this_end_date_time  ; if index(vformat(this_end_date_time),'DATETIME')>0 then this_end_date=datepart(this_end_date_time);
      call symput('valid_time_id','Y');
      putlog %logger(d,info,%bquote(create_time_vtree primary_time_id=&primary_time_id primary_time_level=&primary_time_level)) time_type= time_name= this_start_date= yymmdd10. this_end_date= yymmdd10.;
      format this_start_date_time this_end_date_time datetime19. this_start_date this_end_date yymmdd10.;
   run;
   %if &valid_time_id=N %then
      %do;
         %logger(m,warn,%bquote(primary_time_id=&primary_time_id primary_time_level=&primary_time_level is not a valid time_id/time_level combination in maxapp.lv&primary_time_number.time))
         %goto done;
      %end;
   %do i=&ntlev %to 2 %by -1; /* info for level 1 will be retrieved as the parent level for level 2 */
      /* &this_id is remembered from above, and will be set to &parent_id at the end of this loop */
      %let this_level   =&&tlev&i;
      %let this_number  =&&tln&this_level;
      %let parent_number=&&tpn&this_number;
      %if &parent_number>0 %then
         %do;
            %let parent_level      =&&tnl&parent_number;
            %let grandparent_number=&&tpn&parent_number;
         %end;
      %else
         %do;
            %let parent_level      =0;
            %let grandparent_number=0;
         %end;
      %if &grandparent_number>0 %then
         %do;
            %let grandparent_level =&&tnl&grandparent_number;
         %end;
      %else
         %do;
            %let grandparent_level =0;
         %end;
      /* get the parent id */
      data _parent_id_&this_number;
         set maxapp.lv&this_number.time(where=(lv&this_number.time_lkup_id=&this_id));
         call symput('parent_id',strip(put(lv&parent_number.time_lkup_id,17.0)));
      run;
      /* get the info for the parent id */
      data _parent_id_info_&this_number;
         if 0 then set time_vtree;
         set maxapp.lv&parent_number.time(
                                          where=(time_id=&parent_id)
                                          rename=(
                                                  lv&parent_number.time_lkup_id     =time_id
                                                  lv&parent_number.time_type        =time_type
                                                  lv&parent_number.time_start_date  =this_start_date_time
                                                  lv&parent_number.time_end_date    =this_end_date_time
                                                  %if &grandparent_number>0 %then
                                                     %do;
                                                  lv&grandparent_number.time_lkup_id=time_parent_id
                                                     %end;
                                                  days_in_period                    =this_days_in_period
                                                 )
                                          keep=
                                                  lv&parent_number.time_lkup_id
                                                  lv&parent_number.time_type
                                                  lv&parent_number.time_start_date
                                                  lv&parent_number.time_end_date
                                                  %if &grandparent_number>0 %then
                                                     %do;
                                                  lv&grandparent_number.time_lkup_id
                                                     %end;
                                                  time_name
                                                  days_in_period
                                         );
         parent_row=1;
         time_level=&parent_level;
         time_parent_level=&grandparent_level;
         if time_parent_level<&tlevy then call missing(time_parent_level);
     /* Subhash (S0447415). Check of the date value aganst 22jan2222 is replaced with retrieving data types using vformat syntax --Subhash */ 
         /* ??? ??? the use of vformat below needs to be changed to checking the value of the date variable itself */
         this_start_date=this_start_date_time; if index(vformat(this_start_date_time),'DATETIME')>0 then this_start_date=datepart(this_start_date_time);
         this_end_date  =this_end_date_time  ; if index(vformat(this_end_date_time),'DATETIME')>0 then this_end_date=datepart(this_end_date_time);
         run;
      proc append base=time_vtree data=_parent_id_info_&this_number; run;
      /* get the info for the sibling id's (if any) */
      data _sibling_id_info_&this_number;
         if 0 then set time_vtree;
         set maxapp.lv&this_number.time(
                                        where=(
                                               time_parent_id=&parent_id
                                               and
                                               time_id^=&this_id
                                              )
                                        rename=(
                                                lv&this_number.time_lkup_id   =time_id
                                                lv&this_number.time_type      =time_type
                                                lv&this_number.time_start_date=this_start_date_time
                                                lv&this_number.time_end_date  =this_end_date_time
                                               %if &parent_number>0 %then
                                                  %do;
                                                lv&parent_number.time_lkup_id =time_parent_id
                                                  %end;
                                                days_in_period                =this_days_in_period
                                               )
                                        keep=
                                                lv&this_number.time_lkup_id
                                                lv&this_number.time_type
                                                lv&this_number.time_start_date
                                                lv&this_number.time_end_date
                                               %if &parent_number>0 %then
                                                  %do;
                                                lv&parent_number.time_lkup_id
                                                  %end;
                                                time_name
                                                days_in_period
                                       );
         parent_row=0;
         sibling=1;
         time_level=&this_level;
         time_parent_level=&parent_level;
         if time_parent_level<&tlevy then call missing(time_parent_level);
        /* Subhash (S0447415). Check of the date value aganst 22jan2222 is replaced with retrieving data types using vformat syntax --Subhash */ 
         /* ??? ??? the use of vformat below needs to be changed to checking the value of the date variable itself */
         this_start_date=this_start_date_time; if index(vformat(this_start_date_time),'DATETIME')>0 then this_start_date=datepart(this_start_date_time);
         this_end_date  =this_end_date_time  ; if index(vformat(this_end_date_time),'DATETIME')>0 then this_end_date=datepart(this_end_date_time); 
        run;
      proc append base=time_vtree data=_sibling_id_info_&this_number; run;
      %let this_id=&parent_id; /* will be used the next time through the loop */
   %end;

   /*--------------------------------------------------------------------+
   | note there should not be repeats on time_id for different values of |
   | parent_row,  but we include this here so we can check it later      |
   +--------------------------------------------------------------------*/
   proc sort data=time_vtree nodupkey; by time_level time_id parent_row; run;
   data time_vtree;
      length recomputed base_id sibling 8;
      set time_vtree;
      if ^missing(primary_time_id) then primary_timeid_n=&primary_timeid_n;
      if time_level=&primary_time_level and time_id=&primary_time_id then base_id=1;
      else                                                                base_id=0;
      if base_id=1 or parent_row=1 then recomputed=1;
      else                              recomputed=0;
      if missing(sibling) then sibling=0;
   run;
   data bop_eop_id0;
      merge
         time_vtree(in=in1 where=(time_parent_id is not missing and time_parent_level>=&tlevy))
         sasdata.primary_hasdata_timeids(in=in2 drop=hasdata)
         ;
      by time_level time_id;
      if in1 and in2;
   run;
   data bop_eop_id1(keep=time_level time_id bop_level bop_id eop_level eop_id);
      set bop_eop_id0;
      by time_level time_id;
      if first.time_level then
         do;
            bop_level=time_level;
            bop_id=time_id;
         end;
      if last.time_level then
         do;
            eop_level=time_level;
            eop_id=time_id;
         end;
      if first.time_level or last.time_level then
         do;
            time_id=time_parent_id;
            time_level=time_parent_level;
            output;
         end;
   run;
   proc sort data=bop_eop_id1; by time_level time_id; run;
   data bop_eop_ids;
      merge
         bop_eop_id1(drop=eop_id eop_level where=(bop_id is not missing))
         bop_eop_id1(drop=bop_id bop_level where=(eop_id is not missing))
         ;
      by time_level time_id;
   run;
   data cycle_period(
                     index=(
                            idx_t_li=(time_level time_id)
                           )
                    );
      set
         %do i=1 %to 5;
            maxapp.lv&i.time(
                              in=in&i
                              keep=lv&i.time_lkup_id cycle_id lv&i.time_id
                              rename=(
                                      lv&i.time_lkup_id=time_id
                                      cycle_id         =this_cycle
                                      lv&i.time_id     =this_period
                                     )
                             )
         %end;
         ;
      select;
         %do i=1 %to 5;
            when(in&i) time_level=&&tnl&i;
         %end;
      end;
   run;
   %if &db_type=ORACLE %then
      %do;
         %let _tablename_quoted=%unquote(%str(%')%upcase(&db_tablename&db_tablename_suffix)%str(%'));
         proc sql;
            %connect_to_db(madmax)
            create table oracle_partition_names(index=(high_value)) as
            select * from connection to mmx_db
            (
             select * from all_tab_partitions
             where table_owner='MAXDATA' and table_name=&_tablename_quoted
            )
            order by high_value
            ;
            disconnect from mmx_db;
         quit;
         /*
         proc sort
            data=datamgr.mmax_partitions(where=(upcase(table_name)=upcase("&db_tablename")))
            out=mmax_partitions(
                                keep=partition_name time_level time_id
                                index=(
                                       idx_t_li=(time_level time_id)
                                      )
                               )
            ;
            by time_level time_id;
         run;
         */
         /*-------------------------------------------------------------------------------------------+
         | The following override step is necessary because datamgr.mmax_partitions cannot be trusted |
         +-------------------------------------------------------------------------------------------*/
         data mmax_partitions(keep=partition_name time_level time_id index=(idx_t_li=(time_level time_id)));
            /*
            if 0 then set mmax_partitions;
            */
            /* VJF S0491439 */
            set oracle_partition_names(where=(kverify(ktrim(kleft(kscan(high_value,2,','))),'0123456789')=0));
            time_level=input(scan(high_value,1,','),17.);
            if ^verify(scan(high_value,2,','),'0123456789 ') then
               time_id   =input(scan(high_value,2,','),17.)-1;
            if ^(1<=time_level<=999) or ^(1<=time_id<=9999999999) then delete;
         run;
      %end; /* ORACLE */
   %else %if &db_type=DB2 %then
      %do;
         /* ??? more here for partition names or table names where &db_type=DB2 */
         %let _tablename_quoted=%unquote(%str(%')%upcase(&db_tablename&db_tablename_suffix)%str(%'));
         /* in case of restart, we want to use the same datasets from the previous load */
         %if &mdi_restart ^= Y %then
      	 %do;
         proc sql;
            %connect_to_db(madmax)
            create table db2_partition_names(index=(highvalue)) as
            select * from connection to mmx_db
            (
             select * from syscat.datapartitions
             where tabschema='MAXDATA' and tabname=%bquote(&_tablename_quoted)
            )
            order by highvalue
            ;
            disconnect from mmx_db;
         quit;
         /*
         proc sort
            data=datamgr.mmax_partitions(where=(upcase(table_name)=upcase("&db_tablename")))
            out=mmax_partitions(
                                keep=partition_name time_level time_id
                                index=(
                                       idx_t_li=(time_level time_id)
                                      )
                               )
            ;
            by time_level time_id;
         run;
         */
         /*-------------------------------------------------------------------------------------------+
         | The following override step is necessary because datamgr.mmax_partitions cannot be trusted |
         +-------------------------------------------------------------------------------------------*/
         data sasdata.mmax_partitions(keep=partition_name time_level time_id index=(idx_t_li=(time_level time_id)));
            /*
            if 0 then set mmax_partitions;
            */
            /* VJF S0491439 */
            set db2_partition_names(where=(kverify(ktrim(kleft(kscan(highvalue,2,','))),'+0123456789')>=0)
            rename=(datapartitionname=partition_name));

            time_level=input(scan(translate(lowvalue,'','+.'),1,','),17.);
            if ^verify(scan(lowvalue,2,','),'+0123456789. ') then
               time_id   =input(scan(translate(lowvalue,'','+.'),2,','),17.);
            if ^(1<=time_level<=999) or ^(1<=time_id<=9999999999) then delete;
         run;
         %end; /* mdi_restart */

         data mmax_partitions(index=(idx_t_li=(time_level time_id))); set sasdata.mmax_partitions; run;

         
      %end; /* DB2 */
   %else %if &db_type=SQLSVR %then
      %do;
         /* ??? more here for partition names or table names where &db_type=SQLSVR */
      %end; /* SQLSVR */
   %else %if &db_type=SPDS %then
      %do;
         /* ??? more here for partition names or table names where &db_type=SPDS */
      %end; /* SPDS */
   data &outtab(
                index=(
                       idx_t_li=(time_level time_id)
                      )
               );
      length partition_name $ 80;
      merge time_vtree bop_eop_ids;
      by time_level time_id;
      call missing(this_cycle,this_period);
      set cycle_period key=idx_t_li/unique;
      %if &db_type=ORACLE %then
         %do;
            if 0 then set mmax_partitions;
            call missing(partition_name);
            set mmax_partitions key=idx_t_li/unique;
            partition_name=tranwrd(partition_name,"%upcase(&db_tablename)_","%upcase(&db_tablename&db_tablename_suffix)_");
         %end; /* ORACLE */
      %if &db_type=DB2 %then
         %do;
            if 0 then set mmax_partitions;
            call missing(partition_name);
            set mmax_partitions key=idx_t_li/unique;
            partition_name=tranwrd(partition_name,"%upcase(&db_tablename)_","%upcase(&db_tablename&db_tablename_suffix)_");
         %end; /* DB2 */
      _error_=0;
   run;
   proc print data=&outtab;
      var
         primary_timeid_n
         recomputed sibling
         primary_time_id   primary_time_level primary_timeid_n
         time_level        time_id
         time_parent_level time_parent_id
         parent_row
         bop_level bop_id
         eop_level eop_id
         %if &db_type=ORACLE %then
            %do;
               partition_name
            %end;
         ;
      title "R040: &primary_timeid_n: &primary_time_level &primary_time_id time_vtree";
   run;
   title;

   %done:
   %logger(m,info,%bquote([create_time_vtree] end))
%mend create_time_vtree;

%macro _th;
   %local i l p;
   %save_options(prefix=ctv)
   %do i=1 %to &n_primary_timeids;
      %create_time_vtree(
                         primary_timeid_n  =&i
                        ,primary_time_level=&&primary_time_level_&i
                        ,primary_time_id   =&&primary_time_id_&i
                        ,outtab            =mdi_time_vtree&i
                        )
      options nomprint nonotes nosource nosource2;
   %end;
   %restore_saved_options(prefix=ctv)
   /* in RECLASS, we sometimes have so many input tables below that we run out of memory, so we will do it in stages */
   %save_options(prefix=ctv)
   data mdi_time_vtree;
      merge
         %do i=1 %to &n_primary_timeids;
            mdi_time_vtree&i
         %end;
         ;
      by time_level time_id;
   run;
   %do i=1 %to &n_primary_timeids;
      data mdi_time_vtree;
         merge
            mdi_time_vtree
            mdi_time_vtree&i(
                             keep=
                                time_level time_id
                                recomputed base_id
                                primary_time_level primary_time_id
                             where=(primary_time_id is not missing)
                            )
            mdi_time_vtree&i(
                             keep=
                                time_level time_id
                                parent_row
                             where=(parent_row=1)
                            )
            mdi_time_vtree&i(
                             keep=
                                time_level time_id
                                bop_level bop_id eop_level eop_id
                             where=(bop_level is not missing)
                            )
            mdi_time_vtree&i(
                             keep=
                                time_level time_id
                                primary_timeid_n
                             where=(primary_timeid_n is not missing)
                            )
            %if &db_type=ORACLE %then
               %do;
                  mdi_time_vtree&i(
                                   keep=
                                      time_level time_id
                                      partition_name
                                   where=(partition_name is not missing)
                                  )
               %end;
            %if &db_type=DB2 %then
               %do;
                  mdi_time_vtree&i(
                                   keep=
                                      time_level time_id
                                      partition_name
                                   where=(partition_name is not missing)
                                  )
               %end;
            ;
         by time_level time_id;
      run;
      %logger(m,info,%bquote([create_time_vtree] done merging in mdi_time_vtree&i -- &i of &n_primary_timeids))
      options nonotes nomprint;
   %end;
   %restore_saved_options(prefix=ctv)
   /* if a child time level is recomputed, then its parent level is by definition recomputed as well */
   %do i=&ntlev %to 2 %by -1;
      proc sort
            data=mdi_time_vtree(
                                keep=recomputed time_level time_parent_level time_parent_id
                                where=(time_level=&&&tlev&i and recomputed=1 and time_parent_id is not missing)
                               )
            out=recomputed&i   (
                                drop=time_level
                                rename=(
                                        time_parent_level=time_level
                                        time_parent_id   =time_id
                                       )
                               )
            nodupkey
            ;
         by time_parent_level time_parent_id;
      run;
      data mdi_time_vtree;
         merge
            mdi_time_vtree
            recomputed&i
            ;
         by time_level time_id;
      run;
   %end;
   proc sort data=mdi_time_vtree(keep=recomputed time_parent_level time_parent_id time_level time_id) out=_a;                                                                                            
      by time_parent_level time_parent_id time_level time_id;                                                                                 
   run;  
   /*--------------------------------------------------------------------------------------------------------------+
   | the computing_final_data column will indicate whether this load is going to produce the final totals for      |
   | this time segment, e.g., if we are loading the 5th week of March then this load will produce the final totals |
   | for March and for the 1st quarter, but not for the 1st season or for the year                                 |
   +--------------------------------------------------------------------------------------------------------------*/                                                                                                                                  
   data computing_final_parent_data(keep=time_parent_level time_parent_id index=(idx=(time_parent_level time_parent_id)));                                                                                                                                 
      set _a;                                                                                                                                  
      by time_parent_level time_parent_id time_level time_id;                                                                                 
      if last.time_parent_id then 
         if recomputed=1 then
            output;
   run;
   %global n_partitions; %let n_partitions=0;
   %if &db_type=ORACLE %then
      %do;
         %nobs(mmax_partitions,n_partitions,printlog=n)
      %end; 
   %if &db_type=DB2 %then
      %do;
         %nobs(mmax_partitions,n_partitions,printlog=n)
      %end; 
   data sasdata.mdi_time_vtree(
                               index=(
                                      idx_t_li=(time_level time_id)
                                     )
                              );
      length computing_final_data 8;         
      set mdi_time_vtree;
      set computing_final_parent_data(rename=(time_parent_level=time_level time_parent_id=time_id)) key=idx/unique;
      if time_level=&&tlev&ntlev        then computing_final_data=1;
      else if recomputed=1 and _iorc_=0 then computing_final_data=1;
      else                                   computing_final_data=0;
      _error_=0;
      if base_id then sibling=0;
      %if &db_type=ORACLE %then
         %if &n_partitions>0 %then /* some time id's have partitions */
         %do;
               if missing(partition_name) then put %logger(d,warn,there is no &db_tablename partition for) time_level= time_id=;
         %end;
      %if &db_type=DB2 %then
         %if &n_partitions>0 %then /* some time id's have partitions */
         %do;
               if missing(partition_name) then put %logger(d,fatal,there is no &db_tablename partition for) time_level= time_id=;
         %end;
   run;
   proc print data=sasdata.mdi_time_vtree;
      var
         primary_timeid_n
         recomputed computing_final_data sibling
         primary_time_id   primary_time_level primary_timeid_n
         time_level        time_id
         time_parent_level time_parent_id
         parent_row
         bop_level bop_id
         eop_level eop_id
         %if &db_type=ORACLE %then
            %do;
               partition_name
            %end;
         ;
      title 'R041: mdi_create_time_vtree: sasdata.mdi_time_vtree';
   run;
   title;
   %if &db_type=ORACLE %then
      %do;
         proc print data=mmax_partitions;
            title 'R042: mmax_partitions';
         run;
         title;
      %end;
   %if &db_type=DB2 %then
      %do;
         proc print data=mmax_partitions;
            title 'R042: mmax_partitions';
         run;
         title;
      %end;
   /*
   proc export file="&mdi_saslog_directory&_s_.primary_timeid_list.csv" data=sasdata.primary_timeid_list dbms=csv replace; run;
   proc export file="&mdi_saslog_directory&_s_.mdi_time_vtree.csv"      data=sasdata.mdi_time_vtree      dbms=csv replace; run;
   */
%mend _th;
%_th
%timestamp(e,c_time_vtree)
